from django.db import models
from django.contrib.auth import get_user_model
from crea_app.storages import UthoStorage
from .utils import headshot_form_upload_to, project_upload_to,pan_card_upload_to,event_document_upload_to,gigwork_document_upload_to,press_release_upload_to,castingcall_photos_upload_to
from .utils import cancelled_cheque_upload_to,internship_document_upload_to,apprenticeship_document_upload_to,event_banner_upload_to,castingcall_document_upload_to,media_release_upload_to, cover_image_upload_to, thumbnails_upload_to
CustomUser = get_user_model()
from django.conf import settings
from django.core.exceptions import ValidationError
from userprofile_app.models import Skill




class Project(models.Model):

    CHOICES = [
        ('started', 'Started'),
        ('completed', 'Completed'),
        ('ongoing', 'Ongoing'),
    ]

    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    project_title = models.CharField(max_length=255, null=True, blank=True)
    project_name = models.CharField(max_length=255, null=True, blank=True)
    project_tagline = models.CharField(max_length=255, null=True, blank=True)
    character_name = models.CharField(max_length=255, null=True, blank=True)
    # salary = models.DecimalField(max_digits=10, decimal_places=2)
    role_played = models.CharField(max_length=255, null=True, blank=True)
    project_status = models.CharField(max_length=50, null=True, blank=True)
    progress_percentage = models.IntegerField(null=True, blank=True)
    project_type = models.CharField(max_length=50, null=True, blank=True)
    language = models.CharField(max_length=50, null=True, blank=True)
    description = models.TextField(null=True, blank=True)
    primary_email = models.EmailField(null=True, blank=True)
    secondary_email = models.EmailField(null=True, blank=True)
    hashtags = models.JSONField(null=True, blank=True)  # JSON for hashtags
    social_media_links = models.JSONField(null=True, blank=True) 
    genre = models.CharField(max_length=255, null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=50, default="Ongoing", choices=CHOICES)
    thumbnail_image = models.ImageField(upload_to=thumbnails_upload_to, storage=UthoStorage, null=True, blank=True)
    cover_image = models.ImageField(upload_to=cover_image_upload_to, storage=UthoStorage, null=True, blank=True)
    start_date = models.DateField(null=True)
    end_date = models.DateField(null=True)

    def __str__(self):
        return self.project_title
    
    def validate_publish(self):
        mandatory_fields = [
            "project_title", "project_tagline", "character_name", 
            "role_played", "project_type", "language", "description", 
            "primary_email", "genre"
        ]
        for field in mandatory_fields:
            if not getattr(self, field, None):
                raise ValidationError(f"{field} is required for publishing.")
            
class PressRelease(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name="press_releases")
    image = models.ImageField(upload_to='press_release/')
    uploaded_at = models.DateTimeField(auto_now_add=True)

class MediaRelease(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name="media_releases")
    file = models.FileField(upload_to='media_release/')
    uploaded_at = models.DateTimeField(auto_now_add=True)

    def validate_file_extension(self):
        allowed_extensions = ['png', 'jpg', 'jpeg', 'gif', 'bmp', 'webp', 'mp4', 'avi', 'mov', 'mkv']
        if not self.file.name.lower().endswith(tuple(allowed_extensions)):
            raise ValidationError("Only images and videos are allowed for media releases.")
    
    

class Team(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name="teams")
    team_name = models.CharField(max_length=255)
    created_by = models.ForeignKey(CustomUser,on_delete=models.CASCADE,related_name="created_teams",null=True)  # The user who created this team (casting crew lead)
    role_in_project = models.CharField(max_length=255,null=True)  # Role of the crew in the project (e.g., Casting, Production)
    created_at = models.DateTimeField(auto_now_add=True,null=True)

    def __str__(self):
        return f"{self.team_name} ({self.role_in_project})"



class TeamMember(models.Model):
    team = models.ForeignKey(Team, on_delete=models.CASCADE, related_name="members")
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, null=True)  # Optional: Link to existing users
    name = models.CharField(max_length=255)
    role = models.CharField(max_length=255)  # Role in the team (e.g., Actor, Assistant Director)
    added_by = models.ForeignKey(CustomUser,on_delete=models.CASCADE,related_name="added_team_members",null=True)  # The casting crew member who added this person
    joined_at = models.DateTimeField(auto_now_add=True,null=True)

    def __str__(self):
        return f"{self.name} - {self.role} (Added by {self.added_by.username})"


    
class Award(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='awards')  # Linking Award to Project
    title = models.CharField(max_length=255)  # Award Title
    description = models.TextField(blank=True, null=True)  # Optional description of the award
    date_received = models.DateField(blank=True, null=True)  # Optional date field for when the award was received
    image = models.ImageField(upload_to='awards/images/', blank=True, null=True)
    provider = models.CharField(max_length=255, blank=True, null=True)
    
    def __str__(self):
        return f"{self.title} ({self.project.project_title})"


class Feedback(models.Model):
    project = models.ForeignKey(Project, related_name="feedbacks", on_delete=models.CASCADE)
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    feedback_text = models.TextField()
    rating = models.IntegerField()  # Optional: Rating out of 5 or 10
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Feedback by {self.user} on {self.project}"

class ProjectComment(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='comment')
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='comment')
    content = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f"Comment by {self.user.username} on {self.project.project_title}"


class ProjectReport(models.Model):
    REPORT_CHOICES = [
        ('spam', 'Spam'),
        ('abuse', 'Abuse'),
        ('copyright', 'Copyright Violation'),
        ('other', 'Other'),
    ]
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    reason = models.CharField(max_length=50, choices=REPORT_CHOICES)
    description = models.TextField(blank=True, null=True)
    is_blocked = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'project')  # Prevent duplicate reports from the same user

    def __str__(self):
        return f"{self.user.username} reported {self.project.project_title}"
    
class SavedProject(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE,related_name='saved_projects_forms')
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='saved_by_users_forms')
    saved_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'project')  # Prevent duplicate saves

    def __str__(self):
        return f"{self.user.username} saved {self.project.project_title}"


class FollowedProject(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE,related_name="followed_projects"   )
    project = models.ForeignKey(Project, on_delete=models.CASCADE,related_name="followers")
    followed_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        unique_together = ('user', 'project')  # Prevent duplicate follows

    def __str__(self):
        return f"{self.user.username} follows {self.project.project_title}"



class GigWork(models.Model):
    GIGWORK_TYPE_CHOICES = [
        ('individual', 'Individual'),
        ('project', 'Project'),
    ]

    STATUS_CHOICES = [
        ('opening', 'Opening'),
        ('closed', 'Closed')
    ]
    GIGWORK_AUDITION_TYPE_CHOICES = [
        ('online', 'Online'),
        ('offline', 'Offline')
    ]
    
    gigworks_type = models.CharField(max_length=10, choices=GIGWORK_TYPE_CHOICES)
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    selected_project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='gigworks', null=True, blank=True)
    title = models.CharField(max_length=255)
    short_description = models.TextField()
    work_hours = models.PositiveIntegerField()
    gender= models.CharField(max_length=50,default='Unspecified')
    time = models.CharField(max_length=50,null=True, blank=True)  # e.g., week, month
    age = models.CharField(max_length=20)
    body_type = models.CharField(max_length=255, null=True, blank=True)
    location = models.CharField(max_length=255, null=True, blank=True)
    gigwork_mode = models.CharField(max_length=50, choices=GIGWORK_AUDITION_TYPE_CHOICES)  
    url_link = models.URLField(null=True, blank=True)
    salary = models.DecimalField(max_digits=10, decimal_places=2)
    start_date = models.DateField(null=True, blank=True)
    end_date = models.DateField(null=True, blank=True)
    work_type = models.CharField(max_length=50)  # e.g., freelance, full-time
    skills = models.JSONField()  # List of skill IDs
    required_experience = models.JSONField(default=list)  # List of experiences
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='opening')
    thumbnail = models.ImageField(upload_to='gigwork_thumbnails/', null=True, blank=True)
    logo = models.ImageField(upload_to='gigwork_logos/', null=True, blank=True)

    def __str__(self):
        return self.title

class GigWorkApplication(models.Model):
    STATUS_CHOICES = [
        ('applied', 'Applied'),
        ('selected', 'Selected'),
        ('waiting_list', 'Waiting List'),
        ('save_later', 'Save Later'),
        ('rejected', 'Rejected'),
    ]
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)  # Reference to user applying
    gigwork = models.ForeignKey(GigWork, on_delete=models.CASCADE)  # Reference to the gigwork
    user_message = models.TextField()
    attachments = models.JSONField(null=True, blank=True)  # Storing attachment filenames
    applied_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='applied')

    def __str__(self):
        return f"{self.user.username} applied to {self.gigwork.title}"



class CastingCall(models.Model):

    STATUS_CHOICES = [
        ('opening', 'Opening'),
        ('closed', 'Closed')
    ]
    CASTINGCALL_AUDITION_TYPE_CHOICES = [
        ('online', 'Online'),
        ('offline', 'Offline')
    ]
    CASTINGCALL_TYPE_CHOICES = [
        ('individual', 'Individual'),
        ('project', 'Project'),
    ]

    castingcall_type = models.CharField(max_length=10, choices=CASTINGCALL_TYPE_CHOICES)
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='casting_calls', null=True, blank=True)
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE)
    job_title = models.CharField(max_length=255)
    select_project = models.CharField(max_length=255, null=True, blank=True)
    short_description = models.TextField()
    gender = models.CharField(max_length=50)
    salary = models.DecimalField(max_digits=10, decimal_places=2)
    experience = models.CharField(max_length=100)
    role_type = models.CharField(max_length=50)
    age = models.CharField(max_length=20)
    skills = models.JSONField()  
    height = models.CharField(max_length=20, null=True, blank=True)
    body_type = models.CharField(max_length=255, null=True, blank=True)
    upload_photos = models.ImageField(upload_to=castingcall_photos_upload_to, storage=UthoStorage, null=True, blank=True)
    url_link = models.URLField(null=True, blank=True)
    audition_type = models.CharField(max_length=50, choices=CASTINGCALL_AUDITION_TYPE_CHOICES)  
    castingcall_script = models.FileField(upload_to=castingcall_document_upload_to, storage=UthoStorage, null=True, blank=True)
    script_link = models.URLField(null=True, blank=True)
    location = models.CharField(max_length=255, null=True, blank=True)
    start_date = models.DateField()
    end_date = models.DateField()
    start_time = models.TimeField()
    end_time = models.TimeField()
    created_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='opening')
    thumbnail = models.ImageField(upload_to='castingcall_thumbnails/', null=True, blank=True)
    logo = models.ImageField(upload_to='castingcall_logos/', null=True, blank=True)
    deleted = models.BooleanField(default=False)

    def __str__(self):
        return self.job_title
    



class CastingCallApplication(models.Model):
    casting_call = models.ForeignKey(CastingCall,related_name="applications", on_delete=models.CASCADE)
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE,  related_name="applications")
    team = models.ForeignKey( Team, on_delete=models.CASCADE, related_name="team_join_requests", null=True, blank=True)
    cover_letter = models.TextField(null=True, blank=True)
    uploaded_file = models.FileField(
        upload_to="castingcall/applications/",
        null=True,
        blank=True
    )
    # resume = models.FileField(upload_to=castingcall_document_upload_to, storage=UthoStorage, null=True, blank=True)
    applied_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(
        max_length=50, 
            choices = [
        ('applied', 'Applied'),
        ('selected', 'Selected'),
        ('waiting_list', 'Waiting List'),
        ('save_later', 'Save Later'),
        ('rejected', 'Rejected')
    ],
        default="pending"
    )
    def __str__(self):
        return f"{self.user.username} - {self.casting_call.job_title}"


class BankDetails(models.Model):
    beneficiary_name = models.CharField(max_length=100)
    bank_name = models.CharField(max_length=100)
    account_number = models.CharField(max_length=20)
    ifsc_code = models.CharField(max_length=15)

    def __str__(self):
        return f"{self.beneficiary_name} - {self.bank_name}"

class Uploads(models.Model):  
    pan_card = models.FileField(upload_to=pan_card_upload_to, storage=UthoStorage)
    cancelled_cheque = models.FileField(upload_to=cancelled_cheque_upload_to, storage=UthoStorage)

    def __str__(self):
        return f"Uploads for {self.pan_card.name}"

class EventDetails(models.Model):
    event_title = models.CharField(max_length=200)
    event_overview = models.TextField()
    event_location = models.CharField(max_length=255)
    event_price = models.DecimalField(max_digits=10, decimal_places=2)
    total_tickets = models.IntegerField()
    event_type = models.CharField(max_length=50)
    event_date = models.DateField()
    event_time = models.TimeField()
    event_banner = models.ImageField(upload_to=event_banner_upload_to, storage=UthoStorage, null=True, blank=True)


    def __str__(self):
        return self.event_title

class EventRegistration(models.Model):
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='event_registrations')
    organization_name = models.CharField(max_length=100)
    pan_number = models.CharField(max_length=15)
    address = models.CharField(max_length=255)
    gst_status = models.CharField(max_length=3)
    gst_number = models.CharField(max_length=15)
    host_name = models.CharField(max_length=100)
    email = models.EmailField()  # Changed from enter_email_address for clarity
    mobile_number = models.CharField(max_length=15)
    event_document = models.FileField(upload_to=event_document_upload_to, storage=UthoStorage, null=True, blank=True)


    # One-to-One relationships
    bank_details = models.OneToOneField(BankDetails, on_delete=models.CASCADE, related_name='event_registration')
    uploads = models.ForeignKey(Uploads, on_delete=models.CASCADE)
    event_details = models.OneToOneField(EventDetails, on_delete=models.CASCADE, related_name='event_registration')

    def __str__(self):
        return f"{self.organization_name} - {self.event_details.event_title}"
    


class Headshot(models.Model):
    user = models.OneToOneField(CustomUser, on_delete=models.CASCADE, related_name='headshot')
    headshot = models.ImageField(upload_to=headshot_form_upload_to, blank=True, null=True)  # Directory to store images

    def __str__(self):
        return self.user.username

class Internship(models.Model):

    INTERNSHIP_TYPE_CHOICES = [
        ('full-time', 'Full-time'),
        ('part-time', 'Part-time'),
        ('remote', 'Remote'),
    ]

    GENDER_CHOICES = [
        ('Any', 'Any'),
        ('Male', 'Male'),
        ('Female', 'Female'),
        ('Other', 'Other'),
    ]

    STATUS_CHOICES = [
        ('opening', 'Opening'),
        ('closed', 'Closed')
    ]

    fullname = models.CharField(max_length=255)
    organisation_or_individual = models.CharField(max_length=255)
    profile_title = models.CharField(max_length=255)
    email = models.EmailField()
    job_responsibilities = models.TextField()
    location = models.CharField(max_length=255)
    internship_type = models.CharField(max_length=50, choices=INTERNSHIP_TYPE_CHOICES)
    start_date = models.DateField()
    end_date = models.DateField()
    duration = models.CharField(max_length=255)
    salary = models.DecimalField(max_digits=10, decimal_places=2)
    gender = models.CharField(max_length=10, choices=GENDER_CHOICES, default='Any')
    education_required = models.CharField(max_length=255)
    skills_required = models.ManyToManyField(Skill, related_name='internships')
    skills_experience = models.JSONField(default=list)
    posted_by = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='posted_internships')
    is_active = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True,null=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='opening')
    thumbnail = models.ImageField(upload_to='internship_thumbnails/', null=True, blank=True)
    logo = models.ImageField(upload_to='logos/', null=True, blank=True)
    designation = models.CharField(max_length=255, null=True, blank=True)
    age_required = models.CharField(max_length=255, blank=True, null=True)
    language = models.CharField(max_length=255, blank=True, null=True)
    mobile_number = models.CharField(max_length=15, null=True, blank=True)

    def __str__(self):
        return f"{self.fullname} at {self.organisation_or_individual}"


class InternshipApplication(models.Model):
    STATUS_CHOICES = [
        ('applied', 'Applied'),
        ('selected', 'Selected'),
        ('waiting_list', 'Waiting List'),
        ('save_later', 'Save Later'),
        ('rejected', 'Rejected'),
    ]

    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='internship_applications')
    internship = models.ForeignKey(Internship, on_delete=models.CASCADE, related_name='applications')
    description = models.TextField(blank=True, null=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='applied')
    applied_at = models.DateTimeField(auto_now_add=True)


    def __str__(self):
        return f"{self.user.username} - {self.internship.profile_title, self.id}"

class Apprenticeship(models.Model):
    APPRENTICESHIP_TYPE_CHOICES = [
        ('full-time', 'Full-time'),
        ('part-time', 'Part-time'),
        ('remote', 'Remote'),
    ]

    GENDER_CHOICES = [
        ('Any', 'Any'),
        ('Male', 'Male'),
        ('Female', 'Female'),
        ('Other', 'Other'),
    ]

    STATUS_CHOICES = [
        ('opening', 'Opening'),
        ('closed', 'Closed')
    ]
    
    fullname = models.CharField(max_length=255, blank=True, null=True)
    organisation_or_individual = models.CharField(max_length=255, blank=True, null=True)
    profile_title = models.CharField(max_length=255)
    email = models.EmailField()
    job_responsibilities = models.TextField()
    location = models.CharField(max_length=255, blank=True, null=True)
    apprenticeship_type = models.CharField(max_length=50, choices=APPRENTICESHIP_TYPE_CHOICES)
    start_date = models.DateField()
    end_date = models.DateField()
    duration = models.CharField(max_length=255)
    salary = models.DecimalField(max_digits=10, decimal_places=2)
    gender = models.CharField(max_length=10, choices=GENDER_CHOICES, default='Any')
    education_required = models.CharField(max_length=255, blank=True, null=True)
    skills_required = models.ManyToManyField(Skill, related_name='apprenticeship')
    skills_experience = models.JSONField(default=list)
    posted_by = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='posted_apptrenticeship')
    is_active = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True, blank=True, null=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='opening')
    logo = models.ImageField(upload_to='apprenticeship/logos/', blank=True, null=True)
    thumbnail = models.ImageField(upload_to='apprenticeship/thumbnails/', blank=True, null=True)
    designation = models.CharField(max_length=255, null=True, blank=True)
    age_required = models.CharField(max_length=255, blank=True, null=True)
    language = models.CharField(max_length=255, blank=True, null=True)
    mobile_number = models.CharField(max_length=15, null=True, blank=True)

    def __str__(self):
        return f"{self.fullname} at {self.organisation_or_individual}"
    
class ApprenticeApplication(models.Model):
    STATUS_CHOICES = [
        ('applied', 'Applied'),
        ('selected', 'Selected'),
        ('waiting', 'Waiting'),
        ('saved', 'Save Later'),
        ('rejected', 'Rejected'),
    ]

    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='apprenticeship_applications')
    apprenticeship = models.ForeignKey(Apprenticeship, on_delete=models.CASCADE, related_name='applications')
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default='applied')
    applied_at = models.DateTimeField(auto_now_add=True)
    description = models.TextField(blank=True, null=True)
    


    def __str__(self):
        return f"{self.user.username} - {self.apprenticeship.profile_title, self.id} ({self.status})"
    
class Credit(models.Model):
    project = models.ForeignKey(Project, on_delete=models.CASCADE, related_name='credits')
    user = models.ForeignKey(CustomUser, on_delete=models.CASCADE, related_name='credits')
    role = models.CharField(max_length=100)
    comment = models.CharField(max_length=255,blank=True, null=True)
    status = models.CharField(max_length=20, choices=[
        ('pending', 'Pending'),
        ('accepted', 'Accepted'),
        ('declined', 'Declined'),
    ], default='pending')

    def __str__(self):
        return f"{self.user.username} - {self.role} in {self.project.name}"